package com.ctc.webbean.controller;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;
/*
 *
 * Class:		Comm	
 * Method:		Comm
 * Signature: 	Kevin Halloran
 *
 * Date:		6/1/2004
 * Time:		9:48:25 AM
 */

public class Comm {

	String host;
	BufferedWriter DataOut=null;
	BufferedReader DataIn=null;
	boolean online;
	Connection remote = null;
	UserLogin login;
	Mutex controllerLock = new Mutex();

	public Comm(UserLogin user)
	{
		online = false;
		host = "";
		login = user;
	}

	class Connection {
		Socket remoteSocket = null;
		boolean isSerial;
		BufferedWriter DataOut=null;
		BufferedReader DataIn=null;
		InputStream in;
		String host;
		Comm parent;

		Connection(Comm parent, String host) {
			this.host = host;
			this.parent = parent;
			isSerial = false;
		}

		// report if connection is serial or network
		boolean isSerial() {
			return isSerial;
		}
		boolean open() throws IOException, SocketException {
			// it is network
			remoteSocket = new Socket();
			// wait up to 5 seconds for connection
			remoteSocket.connect(new InetSocketAddress(host, 23), 5000);
			in = remoteSocket.getInputStream();
			DataOut = new BufferedWriter(new OutputStreamWriter(remoteSocket.getOutputStream()));
			DataIn = new BufferedReader(new InputStreamReader(in));
			remoteSocket.setSoTimeout(5000);
			return true;
		}

		// close any existing connection
		void close() throws IOException	{
			// it is network
			if (remoteSocket != null)
			{
				remoteSocket.close();
				remoteSocket = null;
			}
		}
	}

	public void getLock() {
		try
		{
			controllerLock.acquire();
		}
		catch (InterruptedException err)
		{
System.out.println("controller getLock got InterruptedException!!!");
		}
	}
	public void releaseLock() {
		controllerLock.release();
	}

	public String getHost() {
		return host;
	}
	public boolean connect(String host) {
		boolean result = false;
		if (online)
			return false;	// already online
		try
		{
			//Get the address of the host we came from.
			this.host = host;
			//			System.out.println("Init Socket");
			System.out.println("Connecting to :"+host);
			// see if previous connection was left open by accident
			if (DataIn !=null)
			{
				DataIn.close();
				DataIn = null;
			}
			if (DataOut != null)
			{
				DataOut.close();
				DataOut = null;
			}
			if (remote != null)
			{
				remote.close();
				remote = null;
			}
			// start new connection
			System.out.println("Create new connection :"+host);
			// get login information
			if (login == null) {
				System.out.println("Comm, invalid login object, null");
				return false;
			}
			Object info = login.getUserLogin(host);
			// connect and test supplying port no.
			if (info == null)
			{
				// hit the cancel button so exit!
				return false;
			}

			remote = new Connection(this,host);
			remote.open();
			//			System.out.println("Wait data");
			DataOut = remote.DataOut;
			DataIn = remote.DataIn;
			String Instr = null;
			//Read data Title
			//Read CRLF
			readLine();
			//Read CRLF
			readLine();

			System.out.println("Send admin");
			if (!writeLine(login.getUserID(info)+"\r\n",true))
			{
				throw new UnknownHostException();
			}
			System.out.println("Send password");

			if (!writeLine(login.getUserPassword(info)+"\r\n",true))	 // must make this setable
				{
				throw new UnknownHostException();
			}
			// Extra CR LF
			readLine();
			delay(300);
			if (!flushInput())
			{
				throw new UnknownHostException();
			}

			//			System.out.println("Send echo");
			if (!writeLine("echo off\r\n",true))
			{
				System.out.println("echo off failed1");
				throw new UnknownHostException();
			}
			readLine();	// flush extra CR LF on prompt
			//			System.out.println("Send prompt");
			delay(300);
			if (!flushInput())
			{
				throw new UnknownHostException();
			}
			if (!writeLine("prompt off\r\n",false))
			{
				System.out.println("prompt off failed1");
				throw new UnknownHostException();
			}
			System.out.println("connection made");

			delay(300);
			if (!flushInput())
			{
				throw new UnknownHostException();
			}
			//			System.out.println("input flushed");

			online = true;
			result = true;

		}
		catch (UnknownHostException err)
		{
			System.out.println("UnknownHostException1");
			err.printStackTrace(System.err);
		}
		catch (IOException err)
		{
			System.out.println("IOException1");
			err.printStackTrace(System.err);
		}
		catch (InterruptedException err)
		{
			System.out.println("InterruptedException");
			err.printStackTrace(System.err);
		}
		finally
		{
			if (!online)
			{
				System.out.println("Closing connection");
				close();
			}
			else
			{
				result = true;
				;
			}
		}
		return result;
	}

	// Shutdown the socket connection
	public void close() {
		try
		{
			online = false;
			if ((remote != null) && (DataOut != null))
			{
				try {
					// do not call writeChar since it calls us if error
					DataOut.write((char)0x04);
					DataOut.flush();
					delay(100);
				}
				catch(Exception ignored) {
				}
			}
			if (DataIn !=null)
			{
				DataIn.close();
				DataIn = null;
			}
			if (DataOut != null)
			{
				DataOut.close();
				DataOut = null;
			}
			if (remote != null)
			{
				remote.close();
				remote = null;
			}
		}
		catch(IOException e){
		}
	}

	// return our online state
	public boolean getOnlineStatus() {
		return online;
	}

	// flush all data on RX
	public boolean flushInput() {
		boolean result = false;
		try
		{
			if (DataIn != null)
			{
				while (DataIn.ready())
				{
					DataIn.read();
				}
				result = true;
			}
		}
		catch (IOException err)
		{
			System.out.println("IOException3");
			err.printStackTrace(System.err);
		}
		return result;
	}

	// wait for RX data and return line terminated with CRLF
	public String readLine() {
		String Instr=null;
		try {
			if (remote != null)
			{
				Instr = DataIn.readLine();
			}
		}
		catch (SocketTimeoutException err)
		{
			System.out.println("readLine timeout...");
			err.printStackTrace(System.err);
		}
		catch (UnknownHostException err)
		{
			System.out.println("UnknownHostExceptiona");
			err.printStackTrace(System.err);
		}
		catch (IOException err)
		{
			System.out.println("IOExceptionb");
			err.printStackTrace(System.err);
		}
		catch (NullPointerException err)
		{
			System.out.println("NullPointerExceptionc");
		}
		finally
		{
			if (Instr == null)
			{
				close();
			}
		}
		return Instr;
	}
	// wait for RX data and return line terminated with CRLF
	public String readLineExtended(int timeoutLoops) {
		String Instr=null;
		if (timeoutLoops == 0)
		{
			timeoutLoops++;
		}
		while(timeoutLoops > 0)
		{
			try {
				if (remote != null)
				{
					Instr = DataIn.readLine();
					timeoutLoops = 0;
					if (Instr != null)
					{
						break;
					}
				}
			}
			catch (SocketTimeoutException err)
			{
				// this is not an error on extended timeouts
				timeoutLoops--;
				Instr = "";		// simple timeout, different from error
			}
			catch (UnknownHostException err)
			{
				System.out.println("UnknownHostExceptiona");
				err.printStackTrace(System.err);
				timeoutLoops--;
			}
			catch (IOException err)
			{
				System.out.println("IOExceptionb");
				err.printStackTrace(System.err);
				timeoutLoops--;
			}
			catch (NullPointerException err)
			{
				System.out.println("NullPointerExceptionc");
				timeoutLoops--;
			}
			finally
			{
				if ((Instr == null) && timeoutLoops == 0)
				{
					close();
				}
			}
		}
		return Instr;
	}



	// Write a line to the controller, not waiting for echo
	public boolean writeLine(String Instr) {
		return writeLine(Instr,false);
	}
	// Write a line to controller and if echo is true, wait for receipt of line sent
	public boolean writeLine(String Instr, boolean echo) {
		boolean result = false;
		try {
			if (remote != null)
			{
				DataOut.write(Instr);
				DataOut.flush();
				// get rid of echo
				if (echo) {
					DataIn.readLine();
				}
				result = true;
			}
		}
		catch (UnknownHostException err)
		{
			System.out.println("UnknownHostExceptiona");
			err.printStackTrace(System.err);
		}
		catch (IOException err)
		{
			System.out.println("IOExceptionb");
			err.printStackTrace(System.err);
		}
		catch (NullPointerException err)
		{
			System.out.println("NullPointerExceptionc");
		}
		finally
		{
			if (!result)
			{
				close();
			}
		}
		return result;
	}

	// write a single character
	public boolean writeChar(char ch) {
		return writeChar(ch,false);
	}
	public boolean writeChar(char ch,boolean echo) {
		boolean result = false;
		try {
			if (remote != null)
			{
				DataOut.write(ch);
				DataOut.flush();
				// get rid of echo
				if (echo) {
					DataIn.read();
				}
				result = true;
			}
		}
		catch (UnknownHostException err)
		{
			System.out.println("UnknownHostException4");
			err.printStackTrace(System.err);
		}
		catch (IOException err)
		{
			System.out.println("IOException4");
			err.printStackTrace(System.err);
		}
		catch (NullPointerException err)
		{
			System.out.println("NullPointerException4");
		}
		finally
		{
			if (!result)
			{
				close();
			}
		}
		return result;
	}


	// Universal delay routine
	public void delay(int time) throws InterruptedException	{
		Thread.sleep(time);
	}
}
