package com.ctc.webbean.controller;

import java.util.Arrays;
import java.util.Comparator;

class MutexManager
{
	private static  int[]	id_lock = new int[1];
	private static  int		id_pool = 0;

	public static int new_id( )
	{
		int id;
		synchronized( id_lock )	{
			id = id_pool++;		}
		return id;
	}

	/**
	 *	Mutex is given an ID when created. This
	 *	function returns once all of the locks in the incoming
	 *  array have been successfully acquired. Locks are always
	 *  acquired in ascending order of id to attempt to avoid
	 *  deadlock situations.
	 *
	 *	locks -	All of these locks must be acquired before
	 *		acquire_multiple returns.
	 *	timeout -Maximum time to wait to acquire each
	 *		lock (milliseconds). The total time for the multiple
	 *		acquire operation could be (timeout * locks.length).
	 **/

	public static void acquire_multiple( Semaphore[] locks, long timeout )
												throws InterruptedException
	{
		Arrays.sort(locks, new LockComparator() );
		for( int i = 0; i < locks.length; ++i )
			locks[i].acquire( timeout ) ;
	}

	private static class LockComparator implements Comparator
	{	public int compare(Object a, Object b)
		{	return ((Semaphore)a).id() - ((Semaphore)b).id();
		}
		public boolean equals(Object obj)
		{	return obj instanceof LockComparator;
		}
	}
}
