/*******************************************************************************
 * Control Technology Corporation
 * Copyright (c) 2004 Control Technology Corporation.
 * 
 * This software is provided as a reference design only and meant for demonstrative
 * purposes.
 * 
 * THIS SOURCE CODE ('PROGRAM') IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY 
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  THE PROGRAM IS
 * PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY 
 * AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR 
 * CORRECTION.
 *******************************************************************************/
package com.ctccontrol.web.sample;

import javax.swing.*;
import java.net.*;
import com.ctc.webbean.controller.Comm;

// The CommScan class is invoked to both connect to the controller
// and provide a thread that will scan the desired registers, updating
// and array of JTextField objects
public class CommScan extends Thread implements Runnable {

	private Thread thrd;
	MonitorRegisters monitor;  // instance that is using us
	Comm controller = null;		// connection class for controller
	private int refreshRate;
	private String reqMsg = "";		// string to send to controller for registers desired
	int qtyGet=0;	 // set number of registers to monitor

	public CommScan(MonitorRegisters activeMon, int rate) {
		// set class public monitor parent 
		monitor = activeMon;
		if (rate < 250) {
			rate = 250;	// 500 ms or slower is suggested
		}
		this.refreshRate = rate;
		// Attempt a connection to the host
		controller = new Comm(activeMon.getLogin());
		if (controller.connect(activeMon.getHost()))
		{
			// connection successful
			// Startup the monitoring thread
			System.out.println("Starting monitor thread...");
			thrd = new Thread(this);
			thrd.start();
		}
	}

	public boolean getOnlineStatus() {
		return controller.getOnlineStatus();
	}

	public void shutdown() {
		// Kill the communications thread if running
		System.out.println("Shutdown CommScan thread");
		if (thrd != null)
		{
			// kill the thread
			thrd.interrupt();
			thrd = null;
		}
		if (controller != null)
		{
			// close the connection to the host
			controller.getLock();
			controller.close();
			controller.releaseLock();
			controller = null;
		}
	}
	public void lockComm()
	{
		if (controller != null)
		{
			controller.getLock();
		}
	}
	public void unlockComm()
	{
		if (controller != null)
		{
			controller.releaseLock();
		}
	}
	public void setRegistersToMonitor()
	{
		lockComm();  // prevent thread from running
		reqMsg = monitor.getRegList()+"\r\n";
		System.out.println(reqMsg);
		unlockComm();
	}
	// Monitoring Thread
	public void run() {
		boolean loopflag = true;
		System.out.println("Monitoring Thread started");
		String Instr;
		int q;
		int regResult;
		String reg = null;
		boolean locked = false;
		// Wait for widget screen to finish its initializing
		while (!monitor.isRunning()) {
			try {
				Thread.sleep(250);
			} catch (InterruptedException e) {}
		}
		while (!monitor.getMonitoring()) {
			try {
				Thread.sleep(250);
			} catch (InterruptedException e) {}
		}
		controller.flushInput();
		// first build a list of registers to retrieve, if there are any widgets (should be)
		setRegistersToMonitor();
		qtyGet = monitor.getRegArray().size();
		if (qtyGet == 0)
		{
			System.out.println("Nothing to monitor, exiting thread");
			controller.close();
			return;	// nothing to monitor
		}
System.out.println("begin monitor loop, size = "+monitor.getRegArray().size());
System.out.println("monitor regs: "+reqMsg);
		// upon entry monitor string all setup for communications
		// must do that so can change on the fly
		while(loopflag)
		{
			try
			{
				while(loopflag)
				{
					if (locked) {
						controller.releaseLock();
						locked = false;
					}
					// update every N milliseconds
					controller.delay(refreshRate);
					// We are now logged into BlueFusion...
					// must walk the table of entries and fill in data
					if (monitor.getMonitoring())
					{
						controller.getLock();
						locked = true;
						if (!controller.flushInput())
						{
							loopflag = false;
							controller.releaseLock();
							locked = false;
							throw new UnknownHostException();
						}
						// for each widget get current value but as one block random read
						if (!write(reqMsg))
						{
							// error occured
							loopflag = false;
							break;
						}
						for (q = 0;q!=qtyGet;q++)
						{
							// get results
							JTextField actReg = (JTextField)monitor.getRegArray().get(q);
							Instr = controller.readLine();
							if (Instr == null)
							{
								// we lost connection
								JOptionPane.showMessageDialog(monitor,
									"Attempted communication to Controller and timed out.\r\nClose Monitor and re-open.",
									"Read error",
									JOptionPane.ERROR_MESSAGE);
								loopflag = false;
								break;
							}
							// update the cell
							if (!monitor.getMonitoring())
								break;	// all done
							//							String result = Integer.toString(count);
							String[] valueSet = Instr.split(" ",4);
							try
							{
								// first convert to integer to ensure correct data
								regResult= Integer.parseInt(valueSet[2].toString());
								// update the widget, could be anything in the array
								// thus modify as required
								// Also note that you may have to mutex it or listen for changes
								// if allowing the user to edit the box at the same time
								actReg.setText(valueSet[2].toString());
							}
							catch (Exception err_x)
							{
								regResult = -1;
								actReg.setText("-1");
							}
							if (!monitor.getMonitoring())
								break;	// all done
						}
					}
				}
				if (locked) {
					controller.releaseLock();
					locked = false;
				}
			}

			catch (UnknownHostException err)
			{
				System.out.println("UnknownHostException");
				err.printStackTrace(System.err);
			}
			catch (InterruptedException err)
			{
				System.out.println("InterruptedException");
			}
			finally
			{
				if (controller != null)
				{
					controller.close();
				}
			}
			break;	  // exit the thread
		}

	}

	// write value of currently selected row to controller.
	boolean writeValue(String reg,int value)
	{
		// user entering decimal
		controller.getLock();
		if (!controller.writeLine(reg+"="+value+"\r\n",true))
		{
			// error occured
			controller.releaseLock();
			return false;
		}
		controller.releaseLock();
		return true;
	}
	boolean write(String str)
	{
		controller.getLock();
		boolean result = controller.writeLine(str);
		controller.releaseLock();
		return result;
	}

	public Comm getController() {
		return controller;
	}

}
