/*******************************************************************************
 * Control Technology Corporation
 * Copyright (c) 2004 Control Technology Corporation.
 * 
 * This software is provided as a reference design only and meant for demonstrative
 * purposes.
 * 
 * THIS SOURCE CODE ('PROGRAM') IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY 
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  THE PROGRAM IS
 * PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY 
 * AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR 
 * CORRECTION.
 *******************************************************************************/
package com.ctccontrol.web.sample;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import com.ctc.webbean.controller.UserLogin;

public class MonitorRegisters extends JInternalFrame implements InternalFrameListener, ActionListener {
	Color background;
	String host;	// who we are connecting to
	String path;	// network discovery name or "" if none
	boolean running = false;
	private CommScan monSession = null;
	boolean online = false;
	private ArrayList monRegisters = new ArrayList(); // list of registers to monitor
	private boolean monitoring = false;
	// Page refresh rate of all widgets in Ms.
	protected int refreshRate = 500;
	JButton btnStart;
	JButton btnStop;
	UserLogin login;
	JPanel regAccess;
	public MonitorRegisters(UserLogin user, String host, String path) {
		super("Monitor",
			true, //resizable
			true, //closable
			true, //maximizable
			true);//iconifiable
		background = Color.lightGray;
		this.host = host;
		this.path = path;
		login = user;
		// Establish communications if possible
		if (!connectController()) {
			// connection failed
			online = false;
			return;
		}
		// Add the monitor panel to the JInternalFrame
		getContentPane().add(setupMonitorPanel());
		// Create the tab control
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		addInternalFrameListener(this);
		pack();
		validate();
		// update the panel
		setVisible(true);
		online = true;
	}

	public boolean isRunning()
	{
		return online;
	}
	public boolean setMonitoring(boolean state) {
		// do not shutdown the thread
		return setMonitoring(state, false);
	}

	private boolean setMonitoring(boolean state, boolean shutdown) {
		// if first time must start session
		if (state == true) {
			if (monitoring == false) {
				if (!connectController()) {
					return false;
				}
			}
		} else {
			// kill the thread
			if ((monSession != null) && shutdown) {
				monSession.shutdown();
				monSession = null;
			}
		}
		monitoring = state;
		return true;
	}
	
	public String getHost() {
		return host;
	}
	// Return main Login structure so can use old/entered user name and password
	public UserLogin getLogin() {
		return login;
	}

	// Determine if monitoring or not
	public boolean getMonitoring() {
		return monitoring;
	}

	// Get Object list of where to store monitored registers
	public ArrayList getRegArray() {
		return monRegisters;
	}
	
	// Get Registers to Monitor
	public String getRegList() {
		// This list could be more or can be derived in many ways, it is left
		// to the developer to determine how for their application
		// For the moment there should be a one to one relation ship between this
		// and monRegisters ArrayList
		return "13002,1,2";
	}
	private boolean connectController() {
		if (monSession == null) {
			monSession = new CommScan(this, refreshRate);
			if (monSession.getOnlineStatus() == false) {
				// tell user failed to connect
				JOptionPane.showMessageDialog(this,
						"Attempted connection to Controller and timed out or bad ID/password.",
						"Connection error", JOptionPane.ERROR_MESSAGE);
				return false;
			}
		}
		return true;
	}
	
	// Panel creation for Monitoring
	private JPanel setupMonitorPanel()
	{
		// do the layout
		JPanel readwritePanel = new JPanel();
		readwritePanel.setBackground(background);
		readwritePanel.setLayout(new BoxLayout(readwritePanel, BoxLayout.Y_AXIS));
		readwritePanel.add(Box.createRigidArea(new Dimension(0,10)));
		readwritePanel.add(new JLabel("13002"));
		final JTextField txtReg13002 = new JTextField(10);
		readwritePanel.add(txtReg13002);
		txtReg13002.setMaximumSize(new Dimension(300,20));
		monRegisters.add(txtReg13002);
		// ActionListener would have to be implemented if doing edit/write while monitoring
		txtReg13002.setEditable(false);
		readwritePanel.add(new JLabel("1"));
		final JTextField txtReg1 = new JTextField("",10);
		readwritePanel.add(txtReg1);
		txtReg1.setMaximumSize(new Dimension(300,20));
		monRegisters.add(txtReg1);
		txtReg1.setEditable(false);
		readwritePanel.add(new JLabel("2"));
		final JTextField txtReg2 = new JTextField("",10);
		readwritePanel.add(txtReg2);
		txtReg1.setMaximumSize(new Dimension(300,20));
		monRegisters.add(txtReg2);
		txtReg2.setEditable(false);
		readwritePanel.add(Box.createRigidArea(new Dimension(0,10)));
		readwritePanel.add(new JSeparator(SwingConstants.HORIZONTAL));
		readwritePanel.add(Box.createRigidArea(new Dimension(0,10)));

		JPanel btnPanel = new JPanel();
		btnPanel.setBackground(background);
		btnPanel.setLayout(new BoxLayout(btnPanel, BoxLayout.X_AXIS));
		btnPanel.add(Box.createRigidArea(new Dimension(10,10)));
		
		btnStop = new JButton("Stop");
		btnPanel.add(btnStop);
		btnStop.setEnabled(false);
		btnStop.addActionListener(this);
		btnPanel.add(Box.createRigidArea(new Dimension(10,10)));
			
		btnStart = new JButton("Start Monitoring Registers");
		btnPanel.add(btnStart);	
		btnStart.addActionListener(this);
		readwritePanel.add(btnPanel);
		return readwritePanel;
	}
	
	// When Start/Stop buttons pressed enable/disable monitoring
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == btnStop)
		{
			// Disable monitoring
			if (!setMonitoring(false))
			{
				return;
			}
			btnStart.setEnabled(true);
			btnStop.setEnabled(false);
		}
		else if (source == btnStart)
		{
			// Enable monitoring
			if (!setMonitoring(true))
			{
				// failed
				return;
			}
			btnStart.setEnabled(false);
			btnStop.setEnabled(true);
		}
	}
	
	public void showErrorMessageBox(String title, String message)
	{
		JOptionPane.showMessageDialog(this,
			message,
			title,
			JOptionPane.ERROR_MESSAGE);
	}
	
	public void internalFrameClosing(InternalFrameEvent e) {
		System.out.println("Closing");
		if (monSession != null)
		{
			// cleanup on controller connection
			monSession.shutdown();
			monSession = null;
		}
	}
	public void internalFrameClosed(InternalFrameEvent e) {
		System.out.println("Closed");
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
		System.out.println("Iconified");
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameActivated(InternalFrameEvent e) {
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
	}

}
