/*******************************************************************************
 * Control Technology Corporation
 * Copyright (c) 2004 Control Technology Corporation.
 * 
 * This software is provided as a reference design only and meant for demonstrative
 * purposes.
 * 
 * THIS SOURCE CODE ('PROGRAM') IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY 
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  THE PROGRAM IS
 * PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY 
 * AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR 
 * CORRECTION.
 *******************************************************************************/
package com.ctccontrol.web.sample;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import com.ctc.webbean.controller.Comm;
import com.ctc.webbean.controller.UserLogin;

// Class responsible for demonstrating how to read and write registers
public class ReadWriteRegister extends JInternalFrame implements InternalFrameListener {
	Color background;
	String host;	// who we are connecting to
	String path;	// network discovery name or "" if none
	boolean running = false;
	public Comm controller = null;		// connection class for controller
	boolean online = false;
	UserLogin login;
	JPanel regAccess;
	public ReadWriteRegister(UserLogin user, String host, String path) {
		super("Read/Write",
			true, //resizable
			true, //closable
			true, //maximizable
			true);//iconifiable
		background = Color.lightGray;
		this.host = host;
		this.path = path;
		login = user;
		// Establish communications if possible
		controller = new Comm(login);
		// connect to controller and get information
		if (controller.connect(host))
		{
			// connection successful
			online = true;
		}
		else
		{
			System.out.println("Setup connection failed...");
			online = false;
			// failed to connect
			JOptionPane.showMessageDialog(this,
				"Attempted connection to Controller and timed out or bad userID/Password.",
				"Setup Connection error",													  
				JOptionPane.ERROR_MESSAGE);
			return;
		}
		// Create the register panel and add to JInternalFrame
		getContentPane().add(setupRegisterPanel());
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		addInternalFrameListener(this);
		pack();
		validate();
		setVisible(true);
	}

	// isRunning - Returns true if connected to controller, else false
	public boolean isRunning()
	{
		return online;
	}
	
	private JPanel setupRegisterPanel()
	{
		// do the layout
		JPanel readwritePanel = new JPanel();
		readwritePanel.setBackground(background);
		readwritePanel.setLayout(new BoxLayout(readwritePanel, BoxLayout.Y_AXIS));
		readwritePanel.add(Box.createRigidArea(new Dimension(0,10)));
		readwritePanel.add(new JLabel("Register:"));
		final JTextField txtRRegister = new JTextField(10);
		readwritePanel.add(txtRRegister);
		txtRRegister.setMaximumSize(new Dimension(300,20));

		readwritePanel.add(new JLabel("Value Read:"));
		final JTextField txtReadValue = new JTextField("",10);
		readwritePanel.add(txtReadValue);
		txtReadValue.setMaximumSize(new Dimension(300,20));


		readwritePanel.add(Box.createRigidArea(new Dimension(0,10)));

		final JButton btnRead = new JButton("Read Register");
		btnRead.setToolTipText("Click button to read register.");
		readwritePanel.add(btnRead);
		readwritePanel.add(Box.createRigidArea(new Dimension(0,10)));
		readwritePanel.add(new JSeparator(SwingConstants.HORIZONTAL));
		readwritePanel.add(Box.createRigidArea(new Dimension(0,10)));
		btnRead.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// read desired register from controller
				if (!txtRRegister.getText().equals(""))
				{
					if (controller != null)
					{
						txtReadValue.setText(readRegister(txtRRegister.getText()));
					}
				}
				else
				{
					// not set...
					showErrorMessageBox("Invalid Register", "Register field is empty!");
				}
			}
		});
		
		// setup for writing registers
		readwritePanel.add(new JLabel("Register:"));
		final JTextField txtWRegister = new JTextField("",10);
		readwritePanel.add(txtWRegister);
		txtWRegister.setMaximumSize(new Dimension(300,20));
		readwritePanel.add(Box.createRigidArea(new Dimension(0,10)));
		readwritePanel.add(new JLabel("Value to write:"));
		final JTextField txtWriteValue = new JTextField("",10);
		readwritePanel.add(txtWriteValue);
		txtWRegister.setMaximumSize(new Dimension(300,20));
		readwritePanel.add(Box.createRigidArea(new Dimension(0,10)));
		
		final JButton btnWrite = new JButton("Write Register");
		btnWrite.setToolTipText("Click button to write value to register.");
		readwritePanel.add(btnWrite);
		readwritePanel.add(Box.createRigidArea(new Dimension(0,10)));	

		btnWrite.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if ((!txtWRegister.getText().equals("")) && (!txtWriteValue.getText().equals("")) )
				{
					if (controller != null)
					{
						writeRegister(txtWRegister.getText(),txtWriteValue.getText());
					}
				}
				else
				{
					// not set...
					showErrorMessageBox("Invalid Register", "Register field is empty!");
				}
			}
		});

		return readwritePanel;
	}
	
	public void showErrorMessageBox(String title, String message)
	{
		JOptionPane.showMessageDialog(this,
			message,
			title,
			JOptionPane.ERROR_MESSAGE);
	}

	public String readRegister(String reg)
	{
		String Instr = "-1";
		if (!controller.writeLine(reg+"\r\n"))
		{
			// error occured
			return Instr;
		}
		while(true)
		{
			// get results
			Instr = controller.readLine();
			if (Instr == null)
			{
				// we lost connection
				JOptionPane.showMessageDialog(this,
				    "Attempted communication to Controller and timed out.\r\nClose Monitor and re-open.",
				    "Read error",
				    JOptionPane.ERROR_MESSAGE);	
				return "-1";
			}
			return Instr;
		}
	}
	
	public boolean writeRegister(String reg, String value)
	{
		if (!controller.writeLine(reg.toString()+"="+value.toString()+"\r\n",true))
		{
			// error occured
			// we lost connection
			JOptionPane.showMessageDialog(this,
			    "Attempted communication to Controller and timed out.\r\nClose Monitor and re-open.",
			    "Write error",
			    JOptionPane.ERROR_MESSAGE);	
			return false;
		}
		return true;
	}
	
	public void internalFrameClosing(InternalFrameEvent e) {
		System.out.println("Closing");
		if (controller != null)
		{
			// cleanup on controller connection
			controller.close();
			controller = null;
		}
	}
	public void internalFrameClosed(InternalFrameEvent e) {
		System.out.println("Closed");
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
		System.out.println("Iconified");
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameActivated(InternalFrameEvent e) {
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
	}

}
