
/*******************************************************************************
 * Control Technology Corporation
 * Copyright (c) 2004 Control Technology Corporation.
 * 
 * This software is provided as a reference design only and meant for demonstrative
 * purposes.
 * 
 * THIS SOURCE CODE ('PROGRAM') IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY 
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  THE PROGRAM IS
 * PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY 
 * AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR 
 * CORRECTION.
 *******************************************************************************/
// Sample Java Web access to CTC 5200 controllers

package com.ctccontrol.web.sample;
import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import javax.swing.JApplet;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import com.ctc.webbean.controller.UserLogin;

public class SampleWeb extends JApplet {
	public JDesktopPane desktop;
	static public String host;		// our local host
	Color background;
	public MainFrame mainframe;
	Container contentPane;
	UserLogin login;	// Holds user name and password info for each access.
	
	public static void setNativeLookAndFeel() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch(Exception e) {
			System.out.println("Error setting native LAF: " + e);
		}
	}

	public void init() {
		//Make sure we have nice window decorations.
		setNativeLookAndFeel();
		JFrame.setDefaultLookAndFeelDecorated(true);
		// get IP address of host we were served from, should be controller
		// else force the IP address or use auto discovery menus
		host = getDocumentBase().getHost();
		System.out.println("Name = "+getName());
		System.out.println("Host IP = "+host+" length = "+host.length());
		// get out title
		String urlBase = getDocumentBase().getPath();
		System.out.println("urlBase ="+ urlBase);

		login = new UserLogin("Web Login");	// only do this once
		makeContentPane();
		// User will be prompted for username/password whenever connect 
		// but previous entry for that host will be shown
	}
	
	public void destroy() {
		// shutting down so clear setup count
		System.out.println("destroy applet.");
		if (desktop != null)
		{
			JInternalFrame [] frm = desktop.getAllFrames();
			for (int i=0;i!=frm.length;i++)
			{
				System.out.println("dispose frame.");
				frm[i].dispose();
			}
		}
	}
	
	static public String getHost() {
		return host;
	}
	// Return main Login structure
	public UserLogin getLogin() {
		return login;
	}

	public Container makeContentPane() {
		String Instr;

		background = Color.lightGray;
		int inset = 100;
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		//Set up the GUI.
		desktop = new JDesktopPane(); //a specialized layered pane
		desktop.setBackground(Color.white);
		mainframe = new MainFrame(this);

		contentPane = mainframe.getContentPane();
		contentPane.add(desktop);
		//Make dragging a little faster but perhaps uglier.
		desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
		mainframe.setBounds(inset, inset,
			screenSize.width  - inset*2,
			screenSize.height - inset*2);
		mainframe.setVisible(true);
		return mainframe;
	}

	// Main window is a separate JFrame, using internal frames.  Embedding
	// on a web page and not in a JFrame is left to the developer but for
	// the most part there a number of user dependent methods, such as using
	// a JPanel instead of frame and referencing the applet contentPane.
	public class MainFrame extends JFrame 
	{
		SampleWeb mainScreen;
		MainFrame(SampleWeb mainSc)
		{
			super("CTC Sample "+" (Local = HTTP Server)");
			mainScreen = mainSc;
			mainScreen.setBackground(Color.lightGray);

			JMenuBar menuBar = new JMenuBar();
			JMenu menu = new JMenu("Sample");
			menu.setMnemonic(KeyEvent.VK_S);
			
			// Read/Write menu and action
			JMenuItem item = new JMenuItem("Read/Write Register");
			item.setMnemonic(KeyEvent.VK_R);

			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.out.println("Read/Write request");
					OpenReadWriteWindow(login, SampleWeb.getHost(),"");
				}
			});
			menu.add(item);
			
			// Monitor Registers menu and action
			item = new JMenuItem("Monitor Registers");
			item.setMnemonic(KeyEvent.VK_M);

			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.out.println("Monitor request");
					OpenMonitorWindow(login, SampleWeb.getHost(),"");
				}
			});
			menu.add(item);
				
			//quit
			item = new JMenuItem("Quit");
			item.setMnemonic(KeyEvent.VK_Q);

			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.out.println("Quit request");
					quit(MainFrame.this);
				}
			});
			menu.add(item);

			menuBar = new JMenuBar();
			menuBar.add(menu);
					
			setJMenuBar(menuBar);			
			
		}

		// Open a ReadWrite Register Internal Frame and connect to the controller
		public void OpenReadWriteWindow(UserLogin user, String host,String path)
		{
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			ReadWriteRegister regView = new ReadWriteRegister(user, host, path);
			if (regView.isRunning())
			{
				JInternalFrame frameInt = regView;
				frameInt.setVisible(true); //necessary as of 1.3
				desktop.add(frameInt);
				try {
					frameInt.setSelected(true);
				} catch (java.beans.PropertyVetoException e2) {
				}
			}
			else
			{
				regView = null;
			}
			setCursor(null);
		}
		
		// Open a Register Monitor Internal Frame and connect to the controller
		public void OpenMonitorWindow(UserLogin user, String host,String path)
		{
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			MonitorRegisters regView = new MonitorRegisters(user, host, path);
			if (regView.isRunning())
			{
				JInternalFrame frameInt = regView;
				frameInt.setVisible(true); //necessary as of 1.3
				desktop.add(frameInt);
				try {
					frameInt.setSelected(true);
				} catch (java.beans.PropertyVetoException e2) {
				}
			}
			else
			{
				regView = null;
			}
			setCursor(null);
		}

		//This method must be evoked from the event-dispatching thread.
		//Note that if we are an unsigned applet then we just hide the main window
		public void quit(JFrame frame) {
			if (quitConfirmed(frame)) {
				System.out.println("Quitting.");
				try
				{
					System.exit(0);
				}
				catch(Exception err)
				{
					System.out.println("Attempted to exit and got exception = "+err);
					// can't close since not a signed jar and am applet
					frame.setVisible(false);
					frame.setSize(0,0);
				}
			}
			System.out.println("Quit operation not confirmed; staying alive.");
		}

		private boolean quitConfirmed(JFrame frame) {
			String s1 = "Quit";
			String s2 = "Cancel";
			Object[] options = {s1, s2};
			int n = JOptionPane.showOptionDialog(frame,
				"Windows are still open.\nDo you really want to quit?",
				"Quit Confirmation",
				JOptionPane.YES_NO_OPTION,
				JOptionPane.QUESTION_MESSAGE,
				null,
				options,
				s1);
			if (n == JOptionPane.YES_OPTION) {
				return true;
			}
			else {
				return false;
			}
		}

	}
		
}





